/*******************************************************************************
 * Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
 *
 * This file is part of the esxboot project.
 *
 * esxboot is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation version 2 and no later version.
 *
 * esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 ******************************************************************************/

/*
 * com32_private.h -- Com32 compatibility library private declarations
 */

#ifndef COM32_PRIVATE_H_
#define COM32_PRIVATE_H_

#include <string.h>
#include <syslog.h>
#include <boot_services.h>
#include <bios.h>
#include "com32.h"

#define CONSTRUCTOR   __attribute__((constructor))

EXTERN void (*bios_log_callback)(int level, const char *, ...);

#define bios_log(_level_, ...)                                     \
   do {                                                            \
      if (bios_log_callback != NULL) {                             \
         bios_log_callback((_level_), __VA_ARGS__);                \
      }                                                            \
   } while (0)

typedef struct {
   uint8_t major;
   uint8_t minor;
   uint8_t derivative;
   uint8_t drive;
} com32_t;

#define STACK_SIZE       (8 * 1024 * 1024)

EXTERN com32_t com32;

EXTERN char _end[];
EXTERN struct com32_sys_args __com32;

static INLINE void *get_bounce_buffer(void)
{
   return __com32.cs_bounce;
}

static INLINE size_t get_bounce_buffer_size(void)
{
   return (size_t)__com32.cs_bounce_size;
}

static INLINE bool isSyslinux(void) {
   return (com32.derivative == COM32_DERIVATIVE_SYSLINUX);
}

static INLINE bool isExtlinux(void) {
   return (com32.derivative == COM32_DERIVATIVE_EXTLINUX);
}

static INLINE bool isIsolinux(void) {
   return (com32.derivative == COM32_DERIVATIVE_ISOLINUX);
}

static INLINE bool isPxelinux(void)
{
   return (com32.derivative == COM32_DERIVATIVE_PXELINUX);
}

void intcall(uint8_t vector, const com32sys_t *iregs, com32sys_t *oregs);
int intcall_check_CF(uint8_t vector, const com32sys_t *iregs,
                     com32sys_t *oregs);

/*
 * memory.c
 */
int int12_get_memory_size(size_t *lowmem);
int int15_e801(size_t *s1, size_t *s2);
int int15_88(size_t *size);
int int15_e820(e820_range_t *desc, uint32_t *next, uint32_t *desc_size);
int get_memory_map(size_t desc_extra_mem, e820_range_t **mmap, size_t *count);

/*
 * malloc.c
 */
void *realloc(void *ptr, size_t size);

/*
 * disk.c
 */
int get_disk_info(uint8_t drive, disk_t *disk);

/*
 * exec.c
 */
int com32_run_command(const char *command);
int com32_run_default(void);

/*
 * net.c
 */
bool isGPXE(void);

#endif /* !COM32_PRIVATE_H_ */
