/*******************************************************************************
 * Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
 *
 * This file is part of the esxboot project.
 *
 * esxboot is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation version 2 and no later version.
 *
 * esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 ******************************************************************************/

/*
 * intcall.c -- Interrupt call wrappers
 */

#include <string.h>
#include <cpu.h>
#include "com32_private.h"

/*-- intcall -------------------------------------------------------------------
 *
 *      Generic BIOS call wrapper.
 *
 * Parameters
 *      IN vector: interrupt #
 *      IN iregs:  pointer to the input registers structure
 *      IN oregs:  pointer to the output registers structure
 *----------------------------------------------------------------------------*/
void intcall(uint8_t vector, const com32sys_t *iregs, com32sys_t *oregs)
{
   com32sys_t tmpiregs, tmporegs;

   if (iregs == NULL) {
      memset(&tmpiregs, 0, sizeof (com32sys_t));
      iregs = &tmpiregs;
   }

   if (oregs == NULL) {
      oregs = &tmporegs;
   }

   memset(oregs, 0, sizeof (com32sys_t));
   __com32.cs_intcall(vector, iregs, oregs);
}

/*-- intcall_check_CF ----------------------------------------------------------
 *
 *      BIOS call wrapper which returns an error if CF=1 on exit.
 *
 * Parameters
 *      IN vector: interrupt #
 *      IN iregs:  pointer to the input registers structure
 *      IN oregs:  pointer to the output registers structure
 *
 * Results
 *      ERR_SUCCESS, or a generic error status.
 *----------------------------------------------------------------------------*/
int intcall_check_CF(uint8_t vector, const com32sys_t *iregs, com32sys_t *oregs)
{
   com32sys_t tmpregs;

   if (oregs == NULL) {
      oregs = &tmpregs;
   }

   intcall(vector, iregs, oregs);

   if (oregs->eflags.l & EFLAGS_CF) {
      return ERR_UNSUPPORTED;
   }

   return ERR_SUCCESS;
}
