/*******************************************************************************
 * Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
 *
 * This file is part of the esxboot project.
 *
 * esxboot is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation version 2 and no later version.
 *
 * esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 ******************************************************************************/

/*
 * volume.c -- Volume management
 */

#include <string.h>
#include <bootlib.h>
#include "com32_private.h"

/*-- get_boot_dir --------------------------------------------------------------
 *
 *      Get the pathname of the boot directory.
 *
 * Parameter
 *      OUT buffer: pointer to the freshly allocated path
 *
 * Results
 *      ERR_SUCCESS, or a generic error status.
 *----------------------------------------------------------------------------*/
int get_boot_dir(char **buffer)
{
   char *root;

   if (buffer == NULL) {
      return ERR_INVALID_PARAMETER;
   }

   root = strdup("");
   if (root == NULL) {
      return ERR_OUT_OF_RESOURCES;
   }

   *buffer = root;

   return ERR_SUCCESS;
}

/*-- get_boot_disk -------------------------------------------------------------
 *
 *      Get the boot device (hard drive, USB stick, CDROM drive, network
 *      device...) information.
 *
 * Parameter
 *      IN disk: pointer to a disk info structure
 *
 * Results
 *      ERR_SUCCESS, or a generic error status.
 *----------------------------------------------------------------------------*/
int get_boot_disk(disk_t *disk)
{
   return get_disk_info(com32.drive, disk);
}
