/*******************************************************************************
 * Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
 *
 * This file is part of the esxboot project.
 *
 * esxboot is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation version 2 and no later version.
 *
 * esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 ******************************************************************************/

/*
 * sort.c -- Sorting routines
 */

#include <bootlib.h>

/*-- bublle_sort ---------------------------------------------------------------
 *
 *      Classic (non-optimized) bubble sort.
 *      This sorting algorithm is stable (it maintains the relative order of
 *      records with equal comparison keys).
 *
 * Parameters
 *      IN base:   pointer to the table to sort
 *      IN nmemb:  number of entries in the table
 *      IN size:   size of each entry
 *      IN compar: comparison function
 *----------------------------------------------------------------------------*/
void bubble_sort(void *base, size_t nmemb, size_t size,
                 int (*compar)(const void *, const void *))
{
   int swapped;
   char *e1, *e2;
   size_t i;

   if (nmemb > 1) {
      do {
         swapped = 0;
         for (i = 0, e1 = base; i < nmemb - 1; i++, e1 += size) {
            e2 = e1 + size;
            if (compar(e1, e2) > 0) {
               mem_swap(e1, e2, size);
               swapped = 1;
            }
         }
      } while (swapped);
   }
}
