#*******************************************************************************
# Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
#
# This file is part of the esxboot project.
#
# esxboot is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation version 2 and no later version.
#
# esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#*******************************************************************************

#
#-- Makefile utils -------------------------------------------------------------
#
MAKEFLAGS  += -s

ECHO       := @echo
print      = $(ECHO) "[$(BUILDENV) $(1)] $(patsubst $(BUILD_DIR)/%,%,$(2))"

RMDIR      =  $(ECHO) "[RMDIR] $(1)" ; rm -rf $(1)
MKDIR      =  mkdir -p $(1)
UNTGZ      =  $(ECHO) "[UNTARGZ] $(1)" ; tar xzf $(1) -C $(2)

ifdef BUILDENV

#
#-- Environment ----------------------------------------------------------------
#
ifeq ($(BUILDENV),com32)
   FIRMWARE   := bios
   APP_EXT    := .c32
   ARCH       := ia32
else
   FIRMWARE   := uefi
   APP_EXT    := .efi

   ifeq ($(BUILDENV),uefi64)
      ARCH       := em64t
   else
      ifeq ($(BUILDENV),uefi32)
         ARCH       := ia32
      else
         $(error ERROR: $(BUILDENV): Unsupported build environment)
      endif
   endif
endif

#
#-- Tool chain -----------------------------------------------------------------
#
include $(TOPDIR)/env/toolchain.mk

#
#-- Tools ----------------------------------------------------------------------
#
TOOLS_DIR  := $(TOPDIR)/build/tools

ifeq ($(FIRMWARE),bios)
   RELOCS     := $(TOOLS_DIR)/relocs
else
   ifeq ($(ARCH),em64t)
      ELF2EFI    := $(TOOLS_DIR)/elf2efi32
   endif

   ifeq ($(ARCH),ia32)
      ELF2EFI    := $(TOOLS_DIR)/elf2efi64
   endif
endif

#
#-- Compilation flags ----------------------------------------------------------
#
ifeq ($(ARCH),em64t)
   AFLAGS     := -f elf64
   CFLAGS     := -m64 -mno-red-zone
   LDFLAGS    := -m elf_x86_64
endif

ifeq ($(ARCH),ia32)
   AFLAGS     := -f elf32
   CFLAGS     := -m32 -march=i386 -mpreferred-stack-boundary=2
   LDFLAGS    := -m elf_i386
endif

ifeq ($(FIRMWARE),bios)
   CFLAGS     += -fPIE -fno-asynchronous-unwind-tables -D__COM32__
   LDSCRIPT   := $(TOPDIR)/bios/com32/com32.ld
else
   CFLAGS     += -fshort-wchar -fno-dwarf2-cfi-asm
   LDFLAGS    += --defsym _start=efi_main
   LDSCRIPT   := $(TOPDIR)/uefi/uefi.lds
   ifeq ($(ARCH),ia32)
      CFLAGS     += -malign-double
   else
      CFLAGS     += -fpie
   endif
endif

CFLAGS     += -nostdinc -isystem $(GCCINC) -ffreestanding -fno-exceptions    \
                                                                             \
              -Os -fstrength-reduce -ffast-math -fomit-frame-pointer         \
              -msoft-float -finline-limit=2000 -freg-struct-return           \
              -fno-strict-aliasing -falign-functions=0 -falign-jumps=0       \
              -falign-labels=0 -falign-loops=0 -fno-stack-protector -fwrapv  \
                                                                             \
              -W -Wall -Werror -std=c99 -Wwrite-strings -Wstrict-prototypes  \
              -Wpointer-arith -Wdeclaration-after-statement                  \
              -Wvla -Woverlength-strings

LDFLAGS    += -nostdlib -q -S -T $(LDSCRIPT)

#
#-- Files locations ------------------------------------------------------------
#
BUILD_DIR  := $(TOPDIR)/build/$(BUILDENV)

LIB_DIR    := $(BUILD_DIR)/lib
LIBC       := $(LIB_DIR)/c/libc.a
LIBFAT     := $(LIB_DIR)/fat/libfat.a
ZLIB       := $(LIB_DIR)/z/libz.a
LIBGCC     := $(shell $(CC) $(CFLAGS) --print-libgcc)
BOOTLIB    := $(LIB_DIR)/boot/libboot.a
FIRMLIB    := $(LIB_DIR)/$(FIRMWARE)$(ARCH)/lib$(FIRMWARE)$(ARCH).a
ENV_LIB    := $(FIRMLIB) $(LIBFAT) $(LIBC) $(ZLIB) $(LIBGCC)

STDINC     := $(TOPDIR)/libc/include $(TOPDIR)/include
ZLIB_INC   := $(TOPDIR)/zlib
LIBFAT_INC := $(TOPDIR)/libfat
EDK2INC    := $(TOPDIR)/uefi/edk2/MdePkg
UEFIINC    := $(TOPDIR)/uefi $(EDK2INC)/Include $(EDK2INC)/Include/Protocol

ifeq ($(ARCH),em64t)
   UEFIINC    += $(EDK2INC)/Include/X64
endif
ifeq ($(ARCH),ia32)
   UEFIINC    += $(EDK2INC)/Include/Ia32
endif

endif # !BUILDENV