#*******************************************************************************
# Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
#
# This file is part of the esxboot project.
#
# esxboot is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation version 2 and no later version.
#
# esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#*******************************************************************************

#
# rules.mk
#

# Source filenames to object filenames convertion
src_to_obj  =  $(patsubst %.S,%.o,$(patsubst %.s,%.o,$(patsubst %.c,%.o,$(1))))
objects     =  $(addprefix $(2),$(call src_to_obj,$(1)))

get_odir    =  $(if $(findstring lib,$(2)),$(LIB_DIR)/$(1),$(BUILD_DIR)/$(1))
get_target  =  $(if $(findstring lib,$(2)),lib$(1).a,$(1)_$(ARCH)$(APP_EXT))

printcmd    =  $(call print,$(1),$@)

# Build variables
ODIR        := $(call get_odir,$(BASENAME),$(TARGETTYPE))
TARGET      := $(ODIR)/$(call get_target,$(BASENAME),$(TARGETTYPE))
OBJECTS     := $(call objects,$(SRC),$(ODIR)/)
CFLAGS      += $(patsubst %,-D%,$(CDEF)) $(patsubst %,-I%,$(STDINC) $(INC))

# Makefile rules
.SUFFIXES: .c .s .S .o .a .elf $(APP_EXT)
.PRECIOUS: $(OBJECTS) $(TARGET)

$(ODIR)/%.o: %.c
	$(call printcmd,CC)
	$(CC) $(CFLAGS) -c -o $@ $<

$(ODIR)/%.o: %.S
	$(call printcmd,GAS)
	$(CC) $(CFLAGS) -c -o $@ $<

$(ODIR)/%.o: %.s
	$(call printcmd,AS)
	$(AS) $(AFLAGS) -o $@ $<

$(ODIR)/%.a: $(OBJECTS)
	$(call printcmd,AR)
	$(AR) crs $@ $^

.PRECIOUS: $(ODIR)/%.elf
$(ODIR)/%.elf: $(OBJECTS) $(LIBS)
	$(call printcmd,LD)
	$(LD) $(LDFLAGS) -o $@ $^

# UEFI rules
ifeq ($(FIRMWARE),uefi)
ifeq ($(TARGETTYPE),app)
EFISUBSYSTEM := 10
else
ifeq ($(TARGETTYPE),bsdrv)
EFISUBSYSTEM := 11
endif
endif

%$(APP_EXT): %.elf
	$(call printcmd,ELF2EFI)
	$(ELF2EFI) --subsystem=$(EFISUBSYSTEM) $(ELF2EFIFLAGS) $< $@
else
%$(APP_EXT): %.elf
	$(call printcmd,OBJCOPY)
	$(OBJCOPY) -O binary $< $@
	$(call printcmd,RELOCS)
	$(RELOCS) $< >> $@
endif

all: dirs $(TARGET)

dirs:
	$(call MKDIR,$(BUILD_DIR))
	$(call MKDIR,$(ODIR))
