/*******************************************************************************
 * Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
 *
 * This file is part of the esxboot project.
 *
 * esxboot is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation version 2 and no later version.
 *
 * esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 ******************************************************************************/

/*
 * mbr.h -- Master Boot Record partition table data structures
 */

#ifndef MBR_H_
#define MBR_H_

#include <stdint.h>

#define PART_TYPE_EMPTY                 0x0
#define PART_TYPE_FAT12                 0x1
#define PART_TYPE_EXTENDED              0x5
#define PART_TYPE_FAT16                 0x6
#define PART_TYPE_WIN_EXTENDED          0xf
#define PART_TYPE_LINUX_EXTENDED        0x85
#define PART_TYPE_GPT                   0xee

#pragma pack(1)
typedef struct {
   uint8_t flags;
   uint8_t start_head;
   uint16_t start_cylsec;
   uint8_t type;
   uint8_t end_head;
   uint16_t end_cylsec;
   uint32_t start_lba;
   uint32_t sectors_num;
} mbr_part_t;
#pragma pack()

#define PRIMARY_PARTITION_ENTRY(mbr, entrynum)     \
   &((mbr_part_t *)(mbr + 0x1be))[entrynum - 1];

#define PART_IS_PROTECTIVE_MBR(_part_)                        \
   ((_part_)->type == PART_TYPE_GPT &&                        \
    (_part_)->start_lba == 1 &&                               \
    (_part_)->flags == 0)

#endif
