/*******************************************************************************
 * Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
 *
 * This file is part of the esxboot project.
 *
 * esxboot is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation version 2 and no later version.
 *
 * esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 ******************************************************************************/

/*
 * mps.h -- Multi Processor Specification (MPS) definitions
 */

#ifndef MPS_H_
#define MPS_H_

#include <sys/types.h>

#define MP_FLOAT_PTR_SIGNATURE      "_MP_"
#define MP_FLOAT_PTR_SIGNATURE_LEN  4
#define MPS_PARAGRAPH_SIZE          16

#pragma pack(1)
typedef struct {
   char     signature[MP_FLOAT_PTR_SIGNATURE_LEN];     /* "_MP_" */
   uint32_t physptr;             /* Configuration table address  */
   uint8_t  length;              /* Our length (paragraphs)      */
   uint8_t  specification;       /* Specification version        */
   uint8_t  checksum;            /* Checksum (makes sum 0)       */
   uint8_t  feature1;            /* Standard or configuration ?  */
   uint8_t  feature2;            /* Bit7 set for IMCR|PIC        */
   uint8_t  feature3;            /* Unused (0)                   */
   uint8_t  feature4;            /* Unused (0)                   */
   uint8_t  feature5;            /* Unused (0)                   */
} MPFloatingPointer_t;

typedef struct {
   char     signature[4];
   uint16_t length;
   uint8_t  spec;
   uint8_t  checksum;
   char     oem[8];
   char     productid[12];
   uint32_t oemptr;
   uint16_t oemsize;
   uint16_t count;
   uint32_t localapic;
   uint16_t extsize;
   uint8_t  extchecksum;
   uint8_t  reserved;
} MPConfigTable_t;
#pragma pack()

#endif /* !MPS_H_ */
