/*******************************************************************************
 * Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
 *
 * This file is part of the esxboot project.
 *
 * esxboot is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation version 2 and no later version.
 *
 * esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 ******************************************************************************/

/*
 * segment.h -- x86 segmentation data structures
 */

#ifndef SEGMENT_H_
#define SEGMENT_H_

#include <sys/types.h>
#include <compat.h>

/*
 * Global Descriptor Table (GDT)
 */
#define ALIGN_GDT       8

typedef uint64_t  gdt_entry_t;   /* Type of a segment descriptor */

#pragma pack(1)
typedef struct {
   uint16_t limit;            /* sizeof (GDT) - 1 (in bytes) */
   uintptr_t base;            /* GDT base address (must be 8-bytes aligned) */
} gdtr_t;
#pragma pack()

static INLINE
void make_gdtr(gdtr_t *gdtr, uintptr_t gdt, uint16_t nentries)
{
   gdtr->limit = nentries * sizeof (gdt_entry_t) - 1;
   gdtr->base = gdt;
}

/*
 * Far pointers
 */
#define TI_GDT          0     /* Global Descriptor Table indicator */
#define TI_LDT          1     /* Local Descriptor Table indicator */

#define RPL_SUPERVISOR  0
#define RPL_USER        3

#define MAKE_SEGMENT_SELECTOR(_index_, _table_type_, _rpl_)        \
   ((uint16_t)((_index_) << 3) + ((_table_type_) << 2) + (_rpl_))

typedef uint16_t  seg_selector_t;

#pragma pack(1)
typedef struct {
   uint32_t offset;           /* Offset within the segment */
   seg_selector_t segment;    /* Segment selector */
} far_pointer_t;
#pragma pack()

static INLINE
void make_far_pointer(far_pointer_t *fp, seg_selector_t segment, uint32_t offset)
{
   fp->offset = offset;
   fp->segment = segment;
}

#endif /* !SEGMENT_H_ */
