/*******************************************************************************
 * Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
 *
 * This file is part of the esxboot project.
 *
 * esxboot is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation version 2 and no later version.
 *
 * esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 ******************************************************************************/

/*
 * generic.c -- Pattern matching
 */

#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

/*-- dirname -------------------------------------------------------------------
 *
 *      Returns the string up to, but not including, the '/' delimiter. If path
 *      does not contain a slash, dirname() returns ".". If path is a NULL
 *      pointer or points to an empty string, dirname() returns the string "."
 *
 * Parameters
 *      IN path: pointer to the pathname
 *
 * Results
 *      A pointer to the output directory path.
 *
 * Side Effects
 *      Input pathname is modified in place.
 *----------------------------------------------------------------------------*/
char *dirname(char *path)
{
   static char slashdot[4];
   bool slash;
   size_t len;
   char *p;

   slashdot[0] = '/';
   slashdot[1] = '\0';
   slashdot[2] = '.';
   slashdot[3] = '\0';

   /* Return "." for NULL or empty strings */
   if (path == NULL || (*path == '\0')) {
      return &slashdot[2];
   }

   /* Remove trailing slashes */
   p = path + strlen(path) - 1;
   while (p > path && *p == '/') {
      p--;
   }
   /* Discard the basename */
   while (p > path && *p != '/') {
      p--;
   }

   if (p == path) {
      /* Either the dir is "/" or there are no slashes */
      return &slashdot[(*p == '/') ? 0 : 2];
   }
   *p = '\0';

   /* Merge repetitive slashes */
   slash = false;
   len = 0;
   for (p = path; *p != '\0'; p++) {
      if (*p == '/') {
         if (slash) {
            continue;
         }
         slash = true;
      } else {
         if (slash) {
            slash = false;
         }
      }
      path[len++] = *p;
   }

   /* Remove final '/' if any */
   if (len > 1 && path[len - 1] == '/') {
      len--;
   }
   path[len] = '\0';

   return path;
}
