;;******************************************************************************
;; Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
;;
;; This file is part of the esxboot project.
;;
;; esxboot is free software ; you can redistribute it and/or modify it under the
;; terms of the GNU General Public License as published by the Free Software
;; Foundation version 2 and no later version.
;;
;; esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
;; A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program         ; if not, write to the Free Software Foundation, Inc.,
;; 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
;;******************************************************************************

;;
;; trampoline.inc
;;

%ifidn __OUTPUT_FORMAT__, win32
%define IS_64_BIT 0
%elifidn __OUTPUT_FORMAT__, elf32
%define IS_64_BIT 0
%elifidn __OUTPUT_FORMAT__, win64
%define IS_64_BIT 1
%elifidn __OUTPUT_FORMAT__, elf64
%define IS_64_BIT 1
%else
%error "Unsupported output format"
%endif

%if IS_64_BIT
[BITS 64]
%else
[BITS 32]
%endif

;;
;; Multiboot constant
;;
%define MULTIBOOT_MAGIC    0x2BADB002

;;
;; Size of types
;;
%define SIZEOF_PTR         (__BITS__ / 8)       ; sizeof ((void *))
%define SIZEOF_RUNADDR     8                    ; sizeof (run_addr_t)
%define SIZEOF_GDTR        (2 + SIZEOF_PTR)     ; sizeof (gdtr_t)

;;
;; handoff_t internal offsets
;; (Must be updated if the handoff_t structure is modified)
;;

%define OFFSET_STACK       (0 * SIZEOF_RUNADDR)               ; handoff->stack
%define OFFSET_RELOCS      (1 * SIZEOF_RUNADDR)               ; handoff->relocs
%define OFFSET_RELOCATE    (2 * SIZEOF_RUNADDR)               ; handoff->relocate
%define OFFSET_MBI         (3 * SIZEOF_RUNADDR)               ; handoff->mbi
%define OFFSET_KERNEL      (4 * SIZEOF_RUNADDR)               ; handoff->kernel
%define OFFSET_GDTR        (5 * SIZEOF_RUNADDR)               ; handoff->gdtr
%define OFFSET_LJUMP       (5 * SIZEOF_RUNADDR + SIZEOF_GDTR) ; handoff->ljump

;;
;; Long Mode control
;;
%define MSR_EFER           0xc0000080           ; Select EFER MSR
%define MSR_EFER_LME_BIT   8                    ; Long mode enabled if set

;;
;; Paging control
;;
%define CR0_PAGING_BIT     31                   ; Paging enabled if set

;;
;; Segment selectors
;;
%define BOOT_CS            0x08                 ; Index 1 in the GDT (RPL=0)
%define BOOT_DS            0x10                 ; Index 2 in the GDT (RPL=0)

;;
;; Global Descriptor table (GDT)
;;
%define GDT_NULL_DESC            0x0
%define GDT_4GB_FLAT_CODE32_DESC 0x00cf9a000000ffff
%define GDT_4GB_FLAT_DATA32_DESC 0x00cf92000000ffff
