/*******************************************************************************
 * Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
 *
 * This file is part of the esxboot project.
 *
 * esxboot is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation version 2 and no later version.
 *
 * esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 ******************************************************************************/

/*
 * safeboot.h -- safeboot module header file
 */

#ifndef SAFEBOOT_H_
#define SAFEBOOT_H_

#include <stdbool.h>
#include <sys/types.h>
#include <bootlib.h>

#define ASSERT(_cond_)

#define SAFEBOOT_CFG                "/boot.cfg"

#define VMWARE_FAT_UUID_LEN         16

typedef struct {
   char *self_path;
   char *next_loader;
   bool rollback;
   bool verbose;
   framebuffer_t fb;
   bool serial;
   int serial_com;
   int serial_speed;
} safeboot_env_t;

EXTERN safeboot_env_t safeboot;

/*
 * bootbank.c
 */
typedef enum {
   BANK_STATE_UNDEFINED = -1,
   BANK_STATE_VALID     =  0,
   BANK_STATE_UPGRADING =  1,
   BANK_STATE_DIRTY     =  2,
   BANK_STATE_INVALID   =  3
} bank_state_t;

typedef struct {
   int bootstate;
   uint32_t updated;
   char *build;
   int volid;
   unsigned char uuid[VMWARE_FAT_UUID_LEN];
   bool upgrading;
   bool valid;
} bootbank_t;

const char *bootstate_to_str(int bootstate);
int get_boot_bank(bool force_rollback, bootbank_t **bootbank);
void bank_clean(void);

/*
 * chainload.c
 */
int chainload(bootbank_t *bank, int *retval);

/*
 * config.c
 */
int bank_get_config(bootbank_t *bank);
int bank_set_bootstate(bootbank_t *bank, int bootstate);

/*
 * uuid.c
 */
int vmfat_uuid_to_str(const unsigned char *uuid, char **uuid_str);
int vmfat_get_uuid(int volid, void *buffer, size_t buflen);

/*
 * gui.c
 */
bool gui_rollback(void);
int gui_resume_default_boot(void);
int gui_init(void);

#endif /* !SAFEBOOT_H_ */
