/*******************************************************************************
 * Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
 *
 * This file is part of the esxboot project.
 *
 * esxboot is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation version 2 and no later version.
 *
 * esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 ******************************************************************************/

/*
 * efi_private.h -- EFI boot services private header
 */

#ifndef EFI_PRIVATE_H_
#define EFI_PRIVATE_H_

#include <sys/types.h>
#include <efiutils.h>
#include <boot_services.h>
#include <bootlib.h>
#include <fb.h>

#ifndef EFI_CONSOLE_CONTROL_PROTOCOL_GUID
#   include "protocol/ConsoleControl.h"
#endif
#ifndef APPLE_BOOT_VIDEO_PROTOCOL_GUID
#   include "protocol/apple.h"
#endif

/*
 * crc.c
 */
uint32_t efi_crc(void *buffer, size_t buflen);

/*
 * init.c
 */
#define WATCHDOG_DISABLE         0
#define WATCHDOG_DEFAULT_TIMEOUT 300   /* 5 minutes, in seconds */

/*
 * memory.c
 */
EXTERN UINTN MapKey;

int get_memory_map(size_t desc_extra_mem, e820_range_t **e820_mmap,
                   size_t *count, efi_info_t *efi_info);
void mem_init(EFI_MEMORY_TYPE MemType);

/*
 * volume.c
 */
EFI_STATUS get_boot_partition(int volid, EFI_HANDLE *volume);

/*
 * vbe.c
 */
typedef struct {
   uint16_t width;   /* Width in pixels */
   uint16_t height;  /* Height in pixels */
   uint8_t depth;    /* Depth in bits per pixel */
} resolution_t;

#define VBE_BPP(_pxl_)                              \
   ((_pxl_)->RedSize + (_pxl_)->GreenSize +         \
    (_pxl_)->BlueSize + (_pxl_)->RsvdSize)

#define IS_VBE_PXL_15_BIT(_pxl_)                    \
   (((_pxl_)->RedSize   == 5) &&                    \
    ((_pxl_)->GreenSize == 5) &&                    \
    ((_pxl_)->BlueSize  == 5) &&                    \
    ((_pxl_)->RsvdSize  == 0))

#define IS_VBE_PXL_16_BIT(_pxl_)                    \
   (((_pxl_)->RedSize   == 5) &&                    \
    ((_pxl_)->GreenSize == 6) &&                    \
    ((_pxl_)->BlueSize  == 5) &&                    \
    ((_pxl_)->RsvdSize  == 0))

#define IS_VBE_PXL_24_BIT(_pxl_)                    \
   (((_pxl_)->RedSize   == 8) &&                    \
    ((_pxl_)->GreenSize == 8) &&                    \
    ((_pxl_)->BlueSize  == 8) &&                    \
    ((_pxl_)->RsvdSize  == 0))

#define IS_VBE_PXL_32_BIT(_pxl_)                    \
   (((_pxl_)->RedSize   == 8) &&                    \
    ((_pxl_)->GreenSize == 8) &&                    \
    ((_pxl_)->BlueSize  == 8) &&                    \
    ((_pxl_)->RsvdSize  == 8))

#define IS_VBE_PIXEL(_pxl_)                                     \
   (IS_VBE_PXL_15_BIT(_pxl_) || IS_VBE_PXL_16_BIT(_pxl_) ||     \
    IS_VBE_PXL_24_BIT(_pxl_) || IS_VBE_PXL_32_BIT(_pxl_))

void set_pixel_format(pixel32_t *pxl, uint32_t red, uint32_t green,
                      uint32_t blue, uint32_t reserved);
void efi_clean_vbe(void);

/*
 * gop.c
 */
EFI_STATUS gop_get_fb_info(resolution_t *res, framebuffer_t *fb);
EFI_STATUS gop_set_video_mode(unsigned int w, unsigned int h, unsigned int bpp);
EFI_STATUS gop_init(resolution_t **res, unsigned int *n);

/*
 * uga.c
 */
EFI_STATUS uga_get_fb_info(resolution_t *res, framebuffer_t *fb);
EFI_STATUS uga_set_video_mode(unsigned int w, unsigned int h, unsigned int bpp);
EFI_STATUS uga_init(resolution_t **res, unsigned int *n);

#endif /* !EFI_PRIVATE_H_ */
