/*******************************************************************************
 * Copyright (c) 2008-2011 VMware, Inc.  All rights reserved.
 *
 * This file is part of the esxboot project.
 *
 * esxboot is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation version 2 and no later version.
 *
 * esxboot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 ******************************************************************************/

/* uefi.lds -- Linker script for UEFI images
 *
 *    - Image starts at address 0x0
 *    - Image entry point is efi_main()
 *    - The first 4-kb are reserverd for the file headers
 *    - Sections are contiguous and 32-bit aligned
 *
 *   Once linked using this script, the obtained ELF binary still has to be
 *   converted to the UEFI PE/COFF format using the elf2efi program.
 */

ENTRY(efi_main)

SECTIONS {
   HEADERS_SIZE = 0x1000;
   SECTION_ALIGNMENT = 32;

   . = HEADERS_SIZE;

   . = ALIGN(SECTION_ALIGNMENT);
   .text : {
      PROVIDE(__executable_start = .);
      *(.text)
      *(.text.*)
      PROVIDE(_etext = .);
   }

   . = ALIGN(SECTION_ALIGNMENT);
   .rodata : {
      *(.rodata)
      *(.rodata.*)
   }

   . = ALIGN(SECTION_ALIGNMENT);
   .data : {
      *(.data)
      *(.data.*)
   }

   . = ALIGN(SECTION_ALIGNMENT);
   .bss : {
      *(.bss)
      *(.bss.*)
      *(COMMON)
      PROVIDE(_end = .);
   }

   /* Discard other sections */
    /DISCARD/ : {
       *(.comment)
       *(.comment.*)
       *(.note)
       *(.note.*)
       *(.eh_frame)
       *(.eh_frame.*)
       *(.rel)
       *(.rel.*)
       *(.discard)
    }
}
